using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace FileSearch
{
	public partial class Form1 : Form
	{
		/// <summary>
		/// Constructor
		/// </summary>
		public Form1()
		{
			InitializeComponent();

			DriveInfo[] drives = DriveInfo.GetDrives();

			// Add the list of drives to the "Search" menu
			foreach (DriveInfo drive in drives)
			{
				ToolStripMenuItem item = new ToolStripMenuItem();
				item.Text = drive.Name;
				if (drive.Name == @"C:\")
				{
					item.Checked = true;
				}

				// Add the event handler as an inline, anonymous method
				item.Click += delegate(object sender, EventArgs e)
				{
					ToolStripMenuItem menuItem = (ToolStripMenuItem)sender;
					menuItem.Checked = !menuItem.Checked;
				};

				drivesToSearchToolStripMenuItem.DropDownItems.Add(item);
			}
		}

		#region BackgroundWorker Event Handlers

		/// <summary>
		/// This code is executed when the BackgroundWorker instance finishes working
		/// </summary>
		void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			SetUIToReadyState();
		}

		/// <summary>
		/// This code is executed when the BackgroundWorker reports progress 
		/// using the "ReportProgress()" method.
		/// </summary>
		void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
		{
			listBox1.Items.Add(e.UserState);
		}

		/// <summary>
		/// This code is executed when the 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
		{
			string searchString = (string)e.Argument;

			ToolStripItemCollection drivesToSearch = drivesToSearchToolStripMenuItem.DropDownItems;
			for (int i = 0; i < drivesToSearch.Count; i++)
			{
				ToolStripMenuItem item = (ToolStripMenuItem)drivesToSearch[i];
				if (item.Checked)
				{
					DirectoryInfo start = new DirectoryInfo(item.Text);
					Search(searchString, start);
				}
			}
		}

		#endregion

		#region UI Event Handlers

		/// <summary>
		/// When the user presses a key, all current activity must stop
		/// </summary>
		private void textBox1_KeyDown(object sender, KeyEventArgs e)
		{
			StopSearch();
		}

		/// <summary>
		/// When the user releases a key, a new search start; actually the user does not
		/// interact directly with the BackgroundWorker, but rather launches the Timer instance
		/// and then this Timer tries to launch the BackgroundWorker, if it is not busy.
		/// </summary>
		private void textBox1_KeyUp(object sender, KeyEventArgs e)
		{
			LaunchTimer();
		}

		/// <summary>
		/// When the Timer "ticks", every 100 milliseconds, try to start the search.
		/// </summary>
		private void timer1_Tick(object sender, EventArgs e)
		{
			StartSearch();
		}

		/// <summary>
		/// If the "Exit" menu entry is selected, quit the application.
		/// </summary>
		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Exit();
		}

		/// <summary>
		/// If the "Cancel" button is clicked, cancel the current search operation
		/// </summary>
		private void button1_Click(object sender, EventArgs e)
		{
			Cancel();
		}

		/// <summary>
		/// If the "Cancel" menu item is selected, cancel the current search operation
		/// </summary>
		private void cancelCurrentOperationToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Cancel();
		}

		#endregion

		/// <summary>
		/// The Timer instance is responsible of relaunching the BackgroundWorker component. 
		/// This is because you cannot call the RunWorkerAsync() method on a BackgroundWorker
		/// component that is busy, and the timer tries every 100 milliseconds to call this 
		/// method on behalf of the user.
		/// </summary>
		private void LaunchTimer()
		{
			timer1.Enabled = true;
		}

		/// <summary>
		/// This recursive method is the core of the application; it takes a 
		/// </summary>
		/// <param name="searchString">The string being searched</param>
		/// <param name="start">The folder being searched</param>
		/// <remarks>This code does not use "foreach" loops, but "for" loops, in
		/// order to have a better raw search performance ("foreach" loops
		/// generate try/catch statements in the IL code, which are more expensive
		/// in terms of CPU time)</remarks>
		private void Search(string searchString, DirectoryInfo start)
		{
			if (!backgroundWorker1.CancellationPending && searchString.Length > 0)
			{
				FileInfo[] files = start.GetFiles();
				for (int i = 0; i < files.Length; i++)
				{
					FileInfo file = files[i];
					if (file.FullName.ToLower().Contains(searchString.ToLower()))
					{
						// This triggers the ProgressChanged event (see above)
						backgroundWorker1.ReportProgress(1, file.FullName);
					}
				}

				DirectoryInfo[] dirs = start.GetDirectories();
				for (int i = 0; i < dirs.Length; i++)
				{
					// Add folder names to the list, if they match the 
					DirectoryInfo dir = dirs[i];
					if (dir.FullName.ToLower().Contains(searchString.ToLower()))
					{
						backgroundWorker1.ReportProgress(1, dir.FullName);
					}

					// Recursive call to search subdirectories
					Search(searchString, dir);
				}
			}
		}

		/// <summary>
		/// Cancels the current search, if any.
		/// </summary>
		private void Cancel()
		{
			SetUIToReadyState();
			StopSearch();
		}

		/// <summary>
		/// This method changes the state of UI events to show that the application is in "idle" state
		/// </summary>
		private void SetUIToReadyState()
		{
			toolStripStatusLabel1.Text = "Ready";
			toolStripStatusLabel2.Text = string.Format("{0} file(s) found", listBox1.Items.Count);
			button1.Enabled = false;
			cancelCurrentOperationToolStripMenuItem.Enabled = false;
		}

		/// <summary>
		/// This method changes the state of UI events to show that the application is searching for files
		/// </summary>
		private void SetUIToSearchState()
		{
			toolStripStatusLabel1.Text = "Searching...";
			toolStripStatusLabel2.Text = string.Empty;
			button1.Enabled = true;
			cancelCurrentOperationToolStripMenuItem.Enabled = true;
		}

		/// <summary>
		/// This method triggers the BackgroundWorker component to start searching for files
		/// </summary>
		/// <remarks>Testing for String.Length is a faster operation than using the "==" operator
		/// or the Equals method.</remarks>
		private void StartSearch()
		{
			string searchText = textBox1.Text;

			// Do not perform searches if there is nothing to search...
			if (searchText.Length == 0)
			{
				Cancel();
			}
			else
			{
				SetUIToSearchState();
				listBox1.Items.Clear();
				if (!backgroundWorker1.IsBusy)
				{
					timer1.Enabled = false;
					backgroundWorker1.RunWorkerAsync(searchText);
				}
			}
		}

		/// <summary>
		/// This method notifies the BackgroundWorker to stop searching the drive
		/// </summary>
		private void StopSearch()
		{
			if (backgroundWorker1.IsBusy)
			{
				backgroundWorker1.CancelAsync();
			}
		}

		/// <summary>
		/// This method tells the BackgroundWorker to stop searching, and quits the application.
		/// </summary>
		private void Exit()
		{
			StopSearch();
			Application.Exit();
		}
	}
}