using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Kosmaczewski.DistributedApp.BusinessDataEntities;

namespace Kosmaczewski.DistributedApp.BusinessLibrary
{
	/// <summary>
	/// This class contains the methods for accessing the data
	/// in the Northwind database, using a disconnected approach
	/// </summary>
	/// <example>
	/// To use this class:
	/// <code>
	/// using(BusinessDataManager manager = new BusinessDataManager())
	/// {
	///		List&lt;Employee&gt; employees = manager.GetEmployees();
	/// }
	/// </code>
	/// </example>
	public sealed class BusinessDataManager : IDisposable
	{
        #region Private Fields

        private Database m_db;
        private DbConnection m_connection;

        #endregion

        #region Constructor

		/// <summary>
		/// Parameterless constructor.
		/// </summary>
		public BusinessDataManager()
		{
			m_db = DatabaseFactory.CreateDatabase("Northwind");
			m_connection = m_db.CreateConnection();
			m_connection.Open();
        }

        #endregion

        #region Public Methods

        /// <summary>
		/// Gets the list of employees of Northwind
		/// </summary>
		/// <returns>A list of employees</returns>
		public List<Employee> GetEmployees()
		{
			Logger.Write("Entering Business.GetEmployees()");
			List<Employee> employees = new List<Employee>();
			try
			{
				DbCommand command = m_db.GetSqlStringCommand("SELECT * FROM Employees");
				command.Connection = m_connection;

				DbDataReader reader = command.ExecuteReader();
				while (reader.Read())
				{
					Employee employee = new Employee();
					employee.EmployeeId = int.Parse(reader["EmployeeID"].ToString());
					employee.FirstName = reader["FirstName"].ToString();
					employee.LastName = reader["LastName"].ToString();
					employees.Add(employee);
				}
				m_connection.Close();
			}
			catch (Exception ex)
			{
				bool rethrow = ExceptionPolicy.HandleException(ex, "Business Policy");
				if (rethrow)
				{
					throw ex;
				}
			}
			return employees;
		}

		/// <summary>
		/// Gets the list of orders of Northwind
		/// </summary>
		/// <returns>A list of orders instance</returns>
		public List<Order> GetOrders()
		{
			Logger.Write("Entering Business.GetOrders()");
			List<Order> orders = new List<Order>();

			try
			{
				DbCommand command = m_db.GetStoredProcCommand("GetOrders");
				command.Connection = m_connection;

				DbDataReader reader = command.ExecuteReader();
				while (reader.Read())
				{
					Order order = new Order();
					order.CustomerId = reader["CustomerId"].ToString();
					order.EmployeeId = int.Parse(reader["EmployeeId"].ToString());
					order.OrderId = int.Parse(reader["OrderId"].ToString());
					orders.Add(order);
				}
				m_connection.Close();

			}
			catch (Exception ex)
			{
				bool rethrow = ExceptionPolicy.HandleException(ex, "Business Policy");
				if (rethrow)
				{
					throw ex;
				}
			}

			return orders;
		}

		/// <summary>
		/// Returns the specified order
		/// </summary>
		/// <param name="orderId">The ID of the order</param>
		/// <returns>An Order instance</returns>
		public Order GetOrderById(int orderId)
		{
			Order order = new Order();
			try
			{
				DbCommand command = m_db.GetStoredProcCommand("GetOrderById");

				DbParameter param = command.CreateParameter();
				param.Value = orderId;
				param.ParameterName = "@id";
				command.Parameters.Add(param);

				command.Connection = m_connection;

				DbDataReader reader = command.ExecuteReader();
				if (reader.Read())
				{
					order.CustomerId = reader["CustomerId"].ToString();
					order.EmployeeId = int.Parse(reader["EmployeeId"].ToString());
					order.OrderId = int.Parse(reader["OrderId"].ToString());
				}
				m_connection.Close();
			}
			catch (Exception ex)
			{
				bool rethrow = ExceptionPolicy.HandleException(ex, "Business Policy");
				if (rethrow)
				{
					throw ex;
				}
			}

			return order;
		}

		/// <summary>
		/// Gets the list of employees.
		/// </summary>
		/// <param name="employeeId">The ID of the employee sought</param>
		/// <returns>An Employee instance</returns>
		public Employee GetEmployeeById(int employeeId)
		{
			Employee employee = new Employee();
			try
			{
				DbCommand command = m_db.GetSqlStringCommand(String.Format("SELECT * FROM Employees WHERE EmployeeID = {0}", employeeId));
				command.Connection = m_connection;

				DbDataReader reader = command.ExecuteReader();
				if (reader.Read())
				{
					employee.EmployeeId = int.Parse(reader["EmployeeID"].ToString());
					employee.FirstName = reader["FirstName"].ToString();
					employee.LastName = reader["LastName"].ToString();
				}
			}
			catch (Exception ex)
			{
				bool rethrow = ExceptionPolicy.HandleException(ex, "Business Policy");
				if (rethrow)
				{
					throw ex;
				}
			}

			return employee;
        }

        #endregion

        #region IDisposable Members

        void IDisposable.Dispose()
		{
			if (m_connection != null)
			{
				if (m_connection.State == ConnectionState.Open)
				{
					m_connection.Close();
				}
				m_connection.Dispose();
				m_connection = null;
				m_db = null;
			}
		}

		#endregion
	}
}
