using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ClickOnceApplication
{
	public partial class Main : Form
    {
        #region Private Field

        private NotificationCenter notificationCenter = NotificationCenter.DefaultNotificationCenter;

        #endregion

        #region Constructor

        /// <summary>
        /// Constructor
        /// </summary>
		public Main()
		{
			InitializeComponent();

			// Adding observers for notifications
			notificationCenter.AddObserver(this, new NotifyCallback(UpdateCounter), NotificationType.WindowCreation);
			notificationCenter.AddObserver(this, new NotifyCallback(AddTreeNode), NotificationType.WindowCreation);
			notificationCenter.AddObserver(this, new NotifyCallback(AddWindowMenuEntry), NotificationType.WindowCreation);

			notificationCenter.AddObserver(this, new NotifyCallback(UpdateCounter), NotificationType.WindowClosure);
			notificationCenter.AddObserver(this, new NotifyCallback(DeleteTreeNode), NotificationType.WindowClosure);
			notificationCenter.AddObserver(this, new NotifyCallback(DeleteWindowMenuEntry), NotificationType.WindowClosure);

			notificationCenter.AddObserver(this, new NotifyCallback(DataArrivedHandler), NotificationType.DataArrived);

			backgroundWorker1.DoWork += new DoWorkEventHandler(backgroundWorker1_DoWork);
			backgroundWorker1.RunWorkerCompleted += new RunWorkerCompletedEventHandler(backgroundWorker1_RunWorkerCompleted);
        }

        #endregion

        #region Notification Callback Methods

        /// <summary>
        /// Updates the counter in the status bar
        /// </summary>
        private void UpdateCounter(Notification notification)
		{
			toolStripStatusLabel1.Text = String.Format("There are {0} windows", this.MdiChildren.Length);
		}

        /// <summary>
        /// Adds a tree node to the tree pane
        /// </summary>
		private void AddTreeNode(Notification notification)
		{
			TreeNode node = new TreeNode();
			Form sender = (Form)notification.Sender;
			node.Text = sender.Text;
			node.Tag = sender;
			treeView1.Nodes.Add(node);
		}

        /// <summary>
        /// Adds an entry to the "Windows" menu
        /// </summary>
		private void AddWindowMenuEntry(Notification notification)
		{
			if (windowToolStripMenuItem.DropDownItems.Count == 0)
			{
				windowToolStripMenuItem.Enabled = true;
			}
			ToolStripMenuItem item = new ToolStripMenuItem();
			Form sender = (Form)notification.Sender;
			item.Text = sender.Text;
			item.Tag = sender;
			item.Click += new EventHandler(item_Click);
			windowToolStripMenuItem.DropDownItems.Add(item);
		}

        /// <summary>
        /// Deletes a tree node from the tree pane when a window is closed
        /// </summary>
		private void DeleteTreeNode(Notification notification)
		{
			TreeNode nodeToDelete = null;
			foreach (TreeNode node in treeView1.Nodes)
			{
				if (node.Tag == notification.Sender)
				{
					nodeToDelete = node;
					break;
				}
			}

			if (nodeToDelete != null)
			{
				treeView1.Nodes.Remove(nodeToDelete);
			}
		}

        /// <summary>
        /// Deletes the "Windows" menu entry for a window that has been closed
        /// </summary>
        /// <param name="notification"></param>
		private void DeleteWindowMenuEntry(Notification notification)
		{
			ToolStripMenuItem itemToDelete = null;
			foreach (ToolStripMenuItem item in windowToolStripMenuItem.DropDownItems)
			{
				if (item.Tag == notification.Sender)
				{
					itemToDelete = item;
					break;
				}
			}

			if (itemToDelete != null)
			{
				windowToolStripMenuItem.DropDownItems.Remove(itemToDelete);
			}

			if (windowToolStripMenuItem.DropDownItems.Count == 0)
			{
				windowToolStripMenuItem.Enabled = false;
			}
		}

        /// <summary>
        /// Executed when some webservice data arrives to the form
        /// </summary>
        private void DataArrivedHandler(Notification notification)
        {
            toolStripStatusLabel2.ForeColor = Color.Green;
            toolStripStatusLabel2.Text = "Ready";

            DataForm dataForm = new DataForm();
            dataForm.MdiParent = this;
            dataForm.Show();
            dataForm.Text = "Northwind Orders";
            dataForm.DataSource = notification.Parameters["GetOrdersData"];
        }

        #endregion

        #region BackgroundWorker Event Handlers

        /// <summary>
        /// Executes when the BackgroundWorker has finished its job
        /// </summary>
		void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			Hashtable results = new Hashtable();
			results.Add("GetOrdersData", e.Result);
			notificationCenter.Notify(backgroundWorker1, NotificationType.DataArrived, results);
		}

        /// <summary>
        /// Executes when the BackgroundWorker starts a background asynchronous operation
        /// </summary>
        void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
		{
			NorthwindService.Service service = new ClickOnceApplication.NorthwindService.Service();
			NorthwindService.Order [] orders = service.GetOrders();
			e.Result = orders;
        }

        #endregion

        #region UI Event Handlers

        private void retrieveDataToolStripMenuItem_Click(object sender, EventArgs e)
		{
			toolStripStatusLabel2.Text = "Retrieving data...";
			toolStripStatusLabel2.ForeColor = Color.Red;
			backgroundWorker1.RunWorkerAsync();
		}

		private void newToolStripMenuItem_Click(object sender, EventArgs e)
		{
			CreateDocument();
        }

        void item_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            Form form = (Form)item.Tag;
            form.Activate();
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            CreateDocument();
        }

        private void aboutToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            ShowAboutBox();
        }

        private void treeView1_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            Form form = (Form)e.Node.Tag;
            form.Activate();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Quit();
        }

        #endregion

        #region Private Methods
        
        /// <summary>
        /// Creates a new "Document" form and adds it as a child of the current "parent"
        /// </summary>
        private void CreateDocument()
		{
			Form1 newForm = new Form1();
			newForm.MdiParent = this;
			newForm.Text = String.Format("Common Form {0}", this.MdiChildren.Length);
			newForm.Show();
		}

        /// <summary>
        /// Displays the About Box of the application
        /// </summary>
        private void ShowAboutBox()
        {
            AboutBox1 aboutBox = new AboutBox1();
            aboutBox.ShowDialog();
        }

        /// <summary>
        /// Quits the application
        /// </summary>
        /// <remarks>If the BackgroundWorker is working, it tells it to stop</remarks>
        private void Quit()
        {
            if (backgroundWorker1.IsBusy)
            {
                backgroundWorker1.CancelAsync();
            }
            Application.Exit();
        }

        #endregion
    }
}