using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace ClickOnceApplication
{
    /// <summary>
    /// Represents the signature to be used by an observer to be notified of an event
    /// </summary>
	public delegate void NotifyCallback(Notification notification);

    /// <summary>
    /// Represents an object that handles object subscription to events, and dispatches them
    /// </summary>
	public class NotificationCenter
	{
		#region Singleton Design Pattern

        /// <summary>
        /// The static constructor is called before the first instance of a class is created
        /// </summary>
		static NotificationCenter()
		{
			defaultNotificationCenter = new NotificationCenter();
		}

        /// <summary>
        /// Making the constructor private disables any further instantiation by other classes
        /// </summary>
		private NotificationCenter()
		{
		}

		private static NotificationCenter defaultNotificationCenter;

        /// <summary>
        /// This public static property exposes the only possible instance
        /// of NotificationCenter that can exist
        /// </summary>
        public static NotificationCenter DefaultNotificationCenter
        {
            get
            {
                return defaultNotificationCenter;
            }
        }

		#endregion

        #region Private Members

        /// <summary>
        /// Represents a relationship between an observer and an observed event
        /// </summary>
		private class Subscription
		{
			private object m_observer = null;
			private NotificationType m_notificationType;
			private NotifyCallback m_callback;

			public NotifyCallback Callback
			{
				get { return m_callback; }
				set { m_callback = value; }
			}

			public object Observer
			{
				get { return m_observer; }
				set { m_observer = value; }
			}

			public NotificationType NotificationType
			{
				get { return m_notificationType; }
				set { m_notificationType = value; }
			}
		}

        private List<Subscription> m_subscriptions = new List<Subscription>();

        #endregion

        #region Public Methods

        public void AddObserver(object observer, NotifyCallback callback, NotificationType notificationType)
		{
			Subscription newSubscription = new Subscription();
			newSubscription.NotificationType = notificationType;
			newSubscription.Observer = observer;
			newSubscription.Callback = callback;
			m_subscriptions.Add(newSubscription);
		}

		public void RemoveObserver(object observer)
		{
            List<Subscription> temp = new List<Subscription>();
			foreach (Subscription subscription in m_subscriptions)
			{
				if (subscription.Observer == observer)
				{
					temp.Add(subscription);
				}
			}

			foreach (Subscription toDelete in temp)
			{
				m_subscriptions.Remove(toDelete);
			}
        }

        #endregion

        #region Notify Overloads

        public void Notify(object sender, NotificationType notificationType)
		{
			Notification notification = new Notification(sender, notificationType);
			Notify(notification);
		}

		public void Notify(object sender, NotificationType notificationType, Hashtable parameters)
		{
			Notification notification = new Notification(sender, notificationType, parameters);
			Notify(notification);
		}

		public void Notify(Notification notification)
		{
			foreach (Subscription subscription in m_subscriptions)
			{
				if (subscription.NotificationType == notification.NotificationType)
				{
					subscription.Callback(notification);
				}
			}
		}

        #endregion
    }
}
